#include "plugin.h"
#include "stack_detector.h"
#include <string>

// Глобальные переменные
static int g_hPlugin = 0;
static int g_hMenu = 0;
static int g_hMenuAuto = 0;
static bool g_autoAnalysis = false;

// Инициализация плагина
bool pluginit(PLUG_INITSTRUCT* initStruct)
{
    initStruct->pluginVersion = PLUGIN_VERSION;
    initStruct->sdkVersion = PLUG_SDKVERSION;
    strncpy_s(initStruct->pluginName, PLUGIN_NAME, _TRUNCATE);

    // Сохраняем handle плагина
    g_hPlugin = initStruct->pluginHandle;

    // Регистрируем коллбэки
    _plugin_registercallback(g_hPlugin, CB_BREAKPOINT, (CBPLUGIN)CBBREAKPOINT);
    _plugin_registercallback(g_hPlugin, CB_STEPPED, (CBPLUGIN)CBSTEPPED);
    _plugin_registercallback(g_hPlugin, CB_EXCEPTION, (CBPLUGIN)CBEXCEPTION);
    _plugin_registercallback(g_hPlugin, CB_MENUENTRY, (CBPLUGIN)CBMENUENTRY);

    // Создаем глобальный детектор стека
    g_stackDetector = new StackDetector();
    if (!g_stackDetector->Initialize()) {
        _plugin_logprintf("[Stack Detector] Failed to initialize stack detector\n");
        return false;
    }

    // Синхронизируем состояние настроек
    g_autoAnalysis = g_stackDetector->GetAutoAnalysis();

    return true;
}

// Остановка плагина
void plugstop()
{
    // Освобождаем ресурсы детектора
    if (g_stackDetector) {
        delete g_stackDetector;
        g_stackDetector = nullptr;
    }
}

// Настройка плагина
void plugsetup(PLUG_SETUPSTRUCT* setupStruct)
{
    // Добавляем пункт в главное меню
    g_hMenu = setupStruct->hMenu;

    // Создаем подменю
    int hMenuStack = _plugin_menuadd(g_hMenu, "&Stack Detector");
    _plugin_menuaddentry(hMenuStack, MENU_SCAN, "&Scan Current Stack");
    g_hMenuAuto = _plugin_menuaddentry(hMenuStack, MENU_AUTO, "&Auto Analysis");
    _plugin_menuaddentry(hMenuStack, MENU_CLEAR, "&Clear Issues");
    _plugin_menuaddentry(hMenuStack, MENU_HELP, "&Help");

    // Устанавливаем состояние меню в соответствии с загруженными настройками
    _plugin_menuentrysetchecked(g_hMenu, MENU_AUTO, g_autoAnalysis);

    _plugin_logprintf("[Stack Detector] Plugin loaded successfully!\n");
    _plugin_logprintf("[Stack Detector] Use the 'Plugins -> Stack Detector' menu to detect stack corruption.\n");
}

// Callback при точке останова
void CBBREAKPOINT(CBTYPE cbType, PLUG_CB_BREAKPOINT* info)
{
    if (g_stackDetector)
        g_stackDetector->OnBreakpoint(info->breakpoint->addr);
}

// Callback при шаге отладчика
void CBSTEPPED(CBTYPE cbType, PLUG_CB_STEPPED* info)
{
    if (g_stackDetector && g_autoAnalysis)
        g_stackDetector->ScanCurrentStack();
}

// Callback при исключении
void CBEXCEPTION(CBTYPE cbType, PLUG_CB_EXCEPTION* info)
{
    if (g_stackDetector && info->Exception)
        g_stackDetector->OnException((EXCEPTION_RECORD*)info->Exception);
}

// Callback при выборе пункта меню
void CBMENUENTRY(CBTYPE cbType, PLUG_CB_MENUENTRY* info)
{
    switch (info->hEntry)
    {
        case MENU_SCAN:
            if (g_stackDetector) {
                _plugin_logprintf("[Stack Detector] Scanning current stack...\n");
                if (g_stackDetector->ScanCurrentStack()) {
                    _plugin_logprintf("[Stack Detector] Stack issues found! Check log for details.\n");
                } else {
                    _plugin_logprintf("[Stack Detector] No stack issues detected.\n");
                }
            }
            break;

        case MENU_AUTO:
            g_autoAnalysis = !g_autoAnalysis;
            _plugin_menuentrysetchecked(g_hMenu, MENU_AUTO, g_autoAnalysis);
            if (g_stackDetector) {
                g_stackDetector->SetAutoAnalysis(g_autoAnalysis);
                // Настройки автоматически сохраняются в SetAutoAnalysis
            }
            _plugin_logprintf("[Stack Detector] Auto analysis %s\n", g_autoAnalysis ? "enabled" : "disabled");
            break;

        case MENU_CLEAR:
            if (g_stackDetector) {
                g_stackDetector->ClearIssues();
                _plugin_logprintf("[Stack Detector] Issues cleared.\n");
            }
            break;

        case MENU_HELP:
            _plugin_logprintf("\n[Stack Detector] Help:\n");
            _plugin_logprintf("This plugin detects various stack corruption issues:\n");
            _plugin_logprintf("1. Stack Overflow - When stack space is exhausted\n");
            _plugin_logprintf("2. Buffer Overflow - When a buffer on stack is overwritten\n");
            _plugin_logprintf("3. Return Address Corruption - When return address is modified\n");
            _plugin_logprintf("4. Stack Alignment - When stack is not properly aligned\n");
            _plugin_logprintf("5. Stack Frame Error - When frame pointers are corrupted\n\n");
            _plugin_logprintf("Usage:\n");
            _plugin_logprintf("- 'Scan Current Stack' - Manually analyze current stack\n");
            _plugin_logprintf("- 'Auto Analysis' - Automatically scan on every stop\n");
            _plugin_logprintf("- 'Clear Issues' - Reset the list of detected issues\n\n");
            _plugin_logprintf("Note: Settings are automatically saved and will persist between sessions.\n");
            break;
    }
}

// Точка входа DLL
BOOL APIENTRY DllMain(HMODULE hModule, DWORD reason, LPVOID reserved)
{
    switch(reason)
    {
        case DLL_PROCESS_ATTACH:
            break;
        case DLL_PROCESS_DETACH:
            break;
        case DLL_THREAD_ATTACH:
            break;
        case DLL_THREAD_DETACH:
            break;
    }
    return TRUE;
}
