#include "stack_detector.h"
#include "plugin.h"
#include "../pluginsdk/_plugins.h"
#include "../pluginsdk/_dbgfunctions.h"
#include "../pluginsdk/bridgemain.h"  // Add bridge main header
#include "../pluginsdk/TitanEngine/TitanEngine.h"

// Add specific script API includes
#include "../pluginsdk/_scriptapi_register.h"
#include "../pluginsdk/_scriptapi_memory.h"
#include "../pluginsdk/_scriptapi_module.h"
#include "../pluginsdk/_scriptapi_function.h"
#include "../pluginsdk/_scriptapi_stack.h"

// Название секции в настройках
#define SETTINGS_SECTION "StackDetector"

// Глобальные указатели на функции из SDK
static DBGFUNCTIONS g_dbgFunctions;

// Глобальный экземпляр детектора
StackDetector* g_stackDetector = nullptr;

// Simplified structure for stack trace
struct SimpleStackEntry {
    duint addr;
};

StackDetector::StackDetector()
    : m_autoAnalysisEnabled(false)
{
}

StackDetector::~StackDetector()
{
    Shutdown();
}

bool StackDetector::Initialize()
{
    // Получаем функции отладчика
    if (!DbgFunctions())
        return false;

    memcpy(&g_dbgFunctions, DbgFunctions(), sizeof(DBGFUNCTIONS));

    m_issues.clear();
    m_functionBasePointers.clear();

    // Загружаем настройки
    LoadSettings();

    return true;
}

void StackDetector::Shutdown()
{
    // Сохраняем настройки перед выключением
    SaveSettings();

    m_issues.clear();
    m_functionBasePointers.clear();
}

void StackDetector::OnBreakpoint(duint address)
{
    if (m_autoAnalysisEnabled)
        ScanCurrentStack();
}

void StackDetector::OnStep()
{
    if (m_autoAnalysisEnabled)
        ScanCurrentStack();
}

bool StackDetector::GetAutoAnalysis() const
{
    return m_autoAnalysisEnabled;
}

bool StackDetector::SaveSettings()
{
    // Сохраняем настройки автоматического анализа
    if (!BridgeSettingSetUint(SETTINGS_SECTION, "AutoAnalysis", m_autoAnalysisEnabled ? 1 : 0)) {
        _plugin_logprintf("[Stack Detector] Failed to save AutoAnalysis setting\n");
        return false;
    }

    // Применяем сохранение настроек
    if (!BridgeSettingFlush()) {
        _plugin_logprintf("[Stack Detector] Failed to flush settings\n");
        return false;
    }

    return true;
}

bool StackDetector::LoadSettings()
{
    duint value = 0;
    if (BridgeSettingGetUint(SETTINGS_SECTION, "AutoAnalysis", &value)) {
        m_autoAnalysisEnabled = (value != 0);
        _plugin_logprintf("[Stack Detector] Loaded settings: Auto Analysis = %s\n",
                          m_autoAnalysisEnabled ? "enabled" : "disabled");
    } else {
        // Если настройки не найдены, используем значения по умолчанию
        m_autoAnalysisEnabled = false;
    }

    return true;
}

void StackDetector::OnException(EXCEPTION_RECORD* exceptionRecord)
{
    // Для определенных исключений проводим анализ стека
    if (exceptionRecord->ExceptionCode == EXCEPTION_STACK_OVERFLOW ||
        exceptionRecord->ExceptionCode == EXCEPTION_ACCESS_VIOLATION)
    {
        ScanCurrentStack();

        // Специфичная обработка для stack overflow
        if (exceptionRecord->ExceptionCode == EXCEPTION_STACK_OVERFLOW)
        {
            AddIssue(
                StackIssueType::StackOverflow,
                (duint)exceptionRecord->ExceptionAddress,
                "Stack overflow exception detected"
            );
        }
        // Проверка на возможное переполнение буфера
        else if (exceptionRecord->ExceptionCode == EXCEPTION_ACCESS_VIOLATION &&
                 exceptionRecord->NumberParameters >= 2)
        {
            // Если это запись в память
            if (exceptionRecord->ExceptionInformation[0] == 1) {
                duint writeAddress = (duint)exceptionRecord->ExceptionInformation[1];

                // Get register values directly
                duint csp = Script::Register::Get(Script::Register::CSP);
                duint stackBottom = csp;
                duint stackTop = stackBottom + 0x10000; // Примерный размер стека

                // Если адрес находится в диапазоне стека
                if (writeAddress >= stackBottom && writeAddress < stackTop) {
                    AddIssue(
                        StackIssueType::BufferOverflow,
                        (duint)exceptionRecord->ExceptionAddress,
                        "Potential stack buffer overflow detected"
                    );
                }
            }
        }
    }
}

const std::vector<StackIssue>& StackDetector::GetIssues() const
{
    return m_issues;
}

void StackDetector::ClearIssues()
{
    m_issues.clear();
}

void StackDetector::SetAutoAnalysis(bool enabled)
{
    m_autoAnalysisEnabled = enabled;

    // Сохраняем настройки сразу после изменения
    SaveSettings();
}

bool StackDetector::ScanCurrentStack()
{
    bool issuesFound = false;

    // Проверяем различные проблемы со стеком
    issuesFound |= CheckStackOverflow();
    issuesFound |= CheckBufferOverflow();
    issuesFound |= CheckReturnAddressCorruption();
    issuesFound |= CheckStackAlignment();
    issuesFound |= CheckFramePointers();

    return issuesFound;
}

bool StackDetector::CheckStackOverflow()
{
    // Get register values directly
    duint csp = Script::Register::Get(Script::Register::CSP);
    duint cip = Script::Register::Get(Script::Register::CIP);

    // Получаем информацию о границах стека из TEB
    NT_TIB* tib = nullptr;
#ifdef _WIN64
    tib = (NT_TIB*)__readgsqword(0x30);
#else
    tib = (NT_TIB*)__readfsdword(0x18);
#endif

    if (!tib)
        return false;

    duint stackBottom = (duint)tib->StackLimit;
    duint stackTop = (duint)tib->StackBase;
    duint currentSP = csp;

    // Проверяем, находится ли указатель стека в допустимом диапазоне
    if (currentSP < stackBottom || currentSP >= stackTop) {
        AddIssue(
            StackIssueType::StackOverflow,
            cip,
            "Stack pointer outside valid stack range"
        );
        return true;
    }

    // Проверяем, близок ли указатель стека к нижней границе
    duint stackWarningThreshold = 4096; // 4KB предупреждение
    if (currentSP - stackBottom < stackWarningThreshold) {
        AddIssue(
            StackIssueType::StackOverflow,
            cip,
            "Stack is close to overflow (less than 4KB available)"
        );
        return true;
    }

    return false;
}

bool StackDetector::CheckBufferOverflow()
{
    // Этот метод требует более сложного анализа
    // Одна из стратегий - искать "канарейки" (контрольные значения) на стеке

    // Get register values directly
    duint csp = Script::Register::Get(Script::Register::CSP);
    duint cip = Script::Register::Get(Script::Register::CIP);

    // Читаем часть стека для анализа
    unsigned char stackData[1024];
    duint bytesRead = 0;
    if (!Script::Memory::Read(csp, stackData, sizeof(stackData), &bytesRead))
        return false;

    // Ищем паттерны, которые могут указывать на коррупцию
    // Например, повторяющиеся байты или байты в ASCII диапазоне
    bool foundRepeatingPattern = false;
    bool foundAsciiPattern = false;

    // Количество повторяющихся байтов для срабатывания
    const int MIN_REPEAT_COUNT = 10;
    int repeatCount = 1;
    unsigned char lastByte = stackData[0];

    for (int i = 1; i < sizeof(stackData); i++) {
        if (stackData[i] == lastByte) {
            repeatCount++;
            if (repeatCount >= MIN_REPEAT_COUNT) {
                foundRepeatingPattern = true;
                break;
            }
        } else {
            lastByte = stackData[i];
            repeatCount = 1;
        }

        // Проверяем на ASCII строки
        if (stackData[i] >= 32 && stackData[i] <= 126) {
            int asciiCount = 1;
            while (i + asciiCount < sizeof(stackData) &&
                   stackData[i + asciiCount] >= 32 &&
                   stackData[i + asciiCount] <= 126) {
                asciiCount++;
            }

            if (asciiCount >= MIN_REPEAT_COUNT) {
                foundAsciiPattern = true;
                break;
            }

            i += asciiCount - 1;
        }
    }

    if (foundRepeatingPattern || foundAsciiPattern) {
        AddIssue(
            StackIssueType::BufferOverflow,
            cip,
            foundRepeatingPattern ?
                "Potential buffer overflow detected (repeating pattern)" :
                "Potential buffer overflow detected (ASCII pattern)"
        );
        return true;
    }

    return false;
}

bool StackDetector::CheckReturnAddressCorruption()
{
    // Get register values directly
    duint cbp = Script::Register::Get(Script::Register::CBP);
    duint cip = Script::Register::Get(Script::Register::CIP);
    duint returnAddr = 0;

    // Читаем адрес возврата из стека
#ifdef _WIN64
    duint bytesRead = 0;
    if (!Script::Memory::Read(cbp + 8, &returnAddr, sizeof(returnAddr), &bytesRead))
        return false;
#else
    duint bytesRead = 0;
    if (!Script::Memory::Read(cbp + 4, &returnAddr, sizeof(returnAddr), &bytesRead))
        return false;
#endif

    // Проверяем, валидный ли адрес возврата
    // Простой проверкой является проверка, находится ли адрес в диапазоне загруженных модулей
    if (!Script::Memory::IsValidPtr(returnAddr)) {
        AddIssue(
            StackIssueType::ReturnAddressCorruption,
            cip,
            "Return address is not a valid memory location"
        );
        return true;
    }

    // Проверяем, содержит ли адрес возврата допустимую инструкцию
    unsigned char instruction[16];
    duint bytesReadInstr = 0;
    if (!Script::Memory::Read(returnAddr, instruction, sizeof(instruction), &bytesReadInstr))
        return false;

    // Анализ инструкции выходит за рамки этого примера,
    // но можно было бы проверить, является ли это допустимой инструкцией x86/x64

    return false;
}

bool StackDetector::CheckStackAlignment()
{
    // Get register values directly
    duint csp = Script::Register::Get(Script::Register::CSP);
    duint cip = Script::Register::Get(Script::Register::CIP);

    // Проверка выравнивания стека
#ifdef _WIN64
    // В x64 указатель стека должен быть выровнен по 16 байт
    if (csp % 16 != 0) {
        AddIssue(
            StackIssueType::StackAlignment,
            cip,
            "Stack pointer is not 16-byte aligned"
        );
        return true;
    }
#else
    // В x86 указатель стека должен быть выровнен по 4 байта
    if (csp % 4 != 0) {
        AddIssue(
            StackIssueType::StackAlignment,
            cip,
            "Stack pointer is not 4-byte aligned"
        );
        return true;
    }
#endif

    return false;
}

bool StackDetector::CheckFramePointers()
{
    // Get register values directly
    duint cbp = Script::Register::Get(Script::Register::CBP);
    duint csp = Script::Register::Get(Script::Register::CSP);
    duint cip = Script::Register::Get(Script::Register::CIP);

    // Базовый указатель должен указывать на область выше стекового указателя
    if (cbp <= csp) {
        AddIssue(
            StackIssueType::StackFrameError,
            cip,
            "Base pointer is below or equal to stack pointer"
        );
        return true;
    }

    // Получаем имя текущей функции
    char moduleName[MAX_PATH] = {0};
    if (Script::Module::NameFromAddr(cip, moduleName)) {
        duint functionStart = 0;
        Script::Function::FunctionInfo info;
        if (Script::Function::GetInfo(cip, &info)) {
            functionStart = info.rvaStart + Script::Module::BaseFromAddr(cip);
            // Сохраняем базовый указатель для этой функции
            auto it = m_functionBasePointers.find(functionStart);
            if (it == m_functionBasePointers.end()) {
                m_functionBasePointers[functionStart] = cbp;
            } else {
                // Проверяем, изменился ли базовый указатель для этой функции
                if (it->second != cbp) {
                    AddIssue(
                        StackIssueType::StackFrameError,
                        cip,
                        "Base pointer has changed unexpectedly within function"
                    );
                    return true;
                }
            }
        }
    }

    return false;
}

std::vector<duint> StackDetector::GetStackTrace()
{
    std::vector<duint> result;

    // Получаем трассировку стека через SDK
    DBGCALLSTACK stack;
    g_dbgFunctions.GetCallStack(&stack);

    for (int i = 0; i < stack.total; i++) {
        if (i < 100) // Limit to 100 entries
            result.push_back(stack.entries[i].addr);
    }

    return result;
}

void StackDetector::AddIssue(StackIssueType type, duint address, const std::string& description)
{
    // Get register values directly
    duint csp = Script::Register::Get(Script::Register::CSP);
    duint cbp = Script::Register::Get(Script::Register::CBP);

    // Get stack trace for analysis
    std::vector<duint> stackTrace = GetStackTrace();

    // Try to find the root cause address
    duint rootCauseAddress = address;
    std::string rootCauseDescription = description;

    // Attempt to identify the most likely root cause based on issue type
    if (type == StackIssueType::StackOverflow || type == StackIssueType::BufferOverflow) {
        // For stack overflows and buffer overflows, the culprit is often a recursion or a large allocation
        // Look for recursive patterns in the stack trace
        if (stackTrace.size() >= 3) {
            std::map<duint, int> addressCounts;
            for (duint addr : stackTrace) {
                addressCounts[addr]++;
            }

            // Find the most frequently occurring address (indicating recursion)
            duint mostFrequentAddr = 0;
            int maxCount = 1;
            for (auto& pair : addressCounts) {
                if (pair.second > maxCount) {
                    maxCount = pair.second;
                    mostFrequentAddr = pair.first;
                }
            }

            // If we found evidence of recursion (address appears multiple times)
            if (maxCount > 2) {
                rootCauseAddress = mostFrequentAddr;
                rootCauseDescription = "Possible excessive recursion detected - appears " +
                                      std::to_string(maxCount) + " times in call stack";
            }
            // Otherwise check for large stack frames
            else if (stackTrace.size() >= 2) {
                // Get the current function's info to analyze stack frame size
                char moduleName[MAX_MODULE_SIZE] = {0};
                Script::Function::FunctionInfo info;

                if (Script::Module::NameFromAddr(address, moduleName) &&
                    Script::Function::GetInfo(address, &info)) {

                    if (stackTrace.size() >= 2) {
                        duint prevFrameAddr = stackTrace[1]; // Previous frame in call stack

                        // Try to estimate stack usage between frames
                        duint currentFramePtr = cbp;
                        duint prevFrameBasePtr = 0;
                        duint bytesRead = 0;

                        // Attempt to read the previous frame's base pointer from current frame
                        #ifdef _WIN64
                        if (Script::Memory::Read(currentFramePtr, &prevFrameBasePtr, sizeof(prevFrameBasePtr), &bytesRead) &&
                            prevFrameBasePtr != 0) {

                            duint frameSize = currentFramePtr - csp;
                            // If frame size is very large, it may be the cause
                            if (frameSize > 8192) { // 8KB is quite large for a stack frame
                                rootCauseDescription = "Excessive stack usage in frame (" +
                                                      std::to_string(frameSize) + " bytes)";
                            }
                        }
                        #else
                        if (Script::Memory::Read(currentFramePtr, &prevFrameBasePtr, sizeof(prevFrameBasePtr), &bytesRead) &&
                            prevFrameBasePtr != 0) {

                            duint frameSize = currentFramePtr - csp;
                            if (frameSize > 4096) { // 4KB threshold for x86
                                rootCauseDescription = "Excessive stack usage in frame (" +
                                                      std::to_string(frameSize) + " bytes)";
                            }
                        }
                        #endif
                    }
                }
            }
        }
    }
    else if (type == StackIssueType::ReturnAddressCorruption) {
        // For return address corruption, look for buffer manipulations in the current function
        // Often corruption happens because of buffer overruns before the crash

        // Try to examine some memory around the corrupted location
        duint returnAddrLocation = 0;
        #ifdef _WIN64
        returnAddrLocation = cbp + 8;
        #else
        returnAddrLocation = cbp + 4;
        #endif

        // Look for patterns indicating buffer manipulation before this address
        duint dataAroundReturn[10] = {0}; // Check 10 values around the return address
        duint bytesRead = 0;

        if (Script::Memory::Read(returnAddrLocation - sizeof(duint) * 5,
                                dataAroundReturn, sizeof(dataAroundReturn), &bytesRead)) {

            // Check for suspicious patterns (all zeros, repeating values, etc.)
            bool hasRepetitivePattern = false;
            bool hasAsciiPattern = false;

            // Simple heuristics to detect patterns
            for (int i = 1; i < 10; i++) {
                if (dataAroundReturn[i] == dataAroundReturn[i-1] && dataAroundReturn[i] != 0) {
                    hasRepetitivePattern = true;
                }

                // Check if it looks like ASCII data
                if (dataAroundReturn[i] > 0x20 && dataAroundReturn[i] < 0x7F) {
                    hasAsciiPattern = true;
                }
            }

            if (hasRepetitivePattern) {
                rootCauseDescription = "Buffer overrun near return address (repetitive pattern)";
            }
            else if (hasAsciiPattern) {
                rootCauseDescription = "Possible string buffer overrun corrupting stack";
            }
        }
    }

    StackIssue issue;
    issue.type = type;
    issue.address = address;
    issue.description = description;
    issue.stackPtr = csp;
    issue.basePtr = cbp;
    issue.stackTrace = stackTrace;

    m_issues.push_back(issue);

    // Выводим сообщение в лог отладчика
    std::string typeStr;
    switch (type) {
        case StackIssueType::StackOverflow: typeStr = "Stack Overflow"; break;
        case StackIssueType::BufferOverflow: typeStr = "Buffer Overflow"; break;
        case StackIssueType::UseAfterReturn: typeStr = "Use After Return"; break;
        case StackIssueType::ReturnAddressCorruption: typeStr = "Return Address Corruption"; break;
        case StackIssueType::StackAlignment: typeStr = "Stack Alignment"; break;
        case StackIssueType::StackFrameError: typeStr = "Stack Frame Error"; break;
        default: typeStr = "Unknown Issue"; break;
    }

    // Format the address to make it clickable in the x64dbg interface
    char moduleText[MAX_MODULE_SIZE] = "";
    char addressText[64];

    // Try to get the module name for the address
    if (Script::Module::NameFromAddr(address, moduleText)) {
        // Format with module name for clickable link: moduleName.address
        sprintf_s(addressText, "%s.%p", moduleText, (void*)address);
    } else {
        // If module name can't be found, use just the address
        sprintf_s(addressText, "%p", (void*)address);
    }

    // Format the root cause address if different from detection address
    char rootCauseAddressText[64];
    char rootCauseModuleText[MAX_MODULE_SIZE] = "";

    if (rootCauseAddress != address &&
        Script::Module::NameFromAddr(rootCauseAddress, rootCauseModuleText)) {
        sprintf_s(rootCauseAddressText, "%s.%p", rootCauseModuleText, (void*)rootCauseAddress);
    } else if (rootCauseAddress != address) {
        sprintf_s(rootCauseAddressText, "%p", (void*)rootCauseAddress);
    }

    // Get a more detailed explanation for educational purposes
    std::string educationalNote = GetEducationalNoteForIssue(type);

    // Print a visually clear header with consistent formatting
    _plugin_logprintf("\n");
    _plugin_logprintf("╔══════════════════════════════════════════════════════════════════════════════\n");
    _plugin_logprintf("║ 🚨 STACK ISSUE DETECTED: %s\n", typeStr.c_str());
    _plugin_logprintf("╠══════════════════════════════════════════════════════════════════════════════\n");

    // Print issue details
    _plugin_logprintf("║ 📍 Location: %s\n", addressText);
    _plugin_logprintf("║ 🔍 Description: %s\n", description.c_str());
    _plugin_logprintf("║ 📊 Stack Pointer: %p  Base Pointer: %p\n", (void*)csp, (void*)cbp);

    // If we found a different root cause, log it
    if (rootCauseAddress != address || rootCauseDescription != description) {
        if (rootCauseAddress != address) {
            _plugin_logprintf("║ 🔎 ROOT CAUSE likely at %s\n", rootCauseAddressText);
        }
        _plugin_logprintf("║ 🔎 ROOT CAUSE analysis: %s\n", rootCauseDescription.c_str());
    }

    // Add educational note for beginners
    _plugin_logprintf("║\n");
    _plugin_logprintf("║ ℹ️ EXPLANATION: %s\n", educationalNote.c_str());

    // Print stack trace header
    if (type == StackIssueType::StackOverflow ||
        type == StackIssueType::BufferOverflow ||
        type == StackIssueType::ReturnAddressCorruption) {

        // Get stack trace
        DBGCALLSTACK stack;
        g_dbgFunctions.GetCallStack(&stack);

        if (stack.total > 0) {
            _plugin_logprintf("╠══════════════════════════════════════════════════════════════════════════════\n");
            _plugin_logprintf("║ 📚 STACK TRACE (call hierarchy leading to the issue):\n");
            _plugin_logprintf("╠══════════════════════════════════════════════════════════════════════════════\n");

            // Track repetitive patterns in the call stack for analysis
            std::map<duint, int> addrRepetitions;
            std::map<std::string, int> moduleRepetitions;

            // Print legend for symbols used in stack trace
            _plugin_logprintf("║ Legend: 🔍 Current Frame | ⚠️ Likely Root Cause | ↺ Recursive Call\n");
            _plugin_logprintf("║\n");

            for (int i = 0; i < stack.total && i < 20; i++) {
                duint addr = stack.entries[i].addr;
                char moduleText[MAX_MODULE_SIZE] = "";
                bool hasModuleName = Script::Module::NameFromAddr(addr, moduleText);

                // Track repetitions for analysis
                addrRepetitions[addr]++;
                if (hasModuleName) {
                    moduleRepetitions[moduleText]++;
                }

                // Prepare frame prefix with appropriate symbol
                std::string framePrefix = "║ ";

                // Add visual indicators based on frame importance
                if (i == 0) {
                    framePrefix += "🔍"; // Current frame
                } else if (addr == rootCauseAddress) {
                    framePrefix += "⚠️"; // Root cause
                } else if (addrRepetitions[addr] > 1) {
                    framePrefix += "↺ "; // Recursive call
                } else {
                    framePrefix += "  "; // Regular frame
                }

                // Format the frame for display
                char frameText[256];
                if (hasModuleName) {
                    sprintf_s(frameText, "[%2d] %s.%p", i, moduleText, (void*)addr);
                } else {
                    sprintf_s(frameText, "[%2d] %p", i, (void*)addr);
                }

                // Additional markers for special frames
                std::string marker = "";
                if (addr == rootCauseAddress) {
                    marker = " ← LIKELY ROOT CAUSE";
                } else if (addrRepetitions[addr] > 1) {
                    marker = " ← RECURSIVE CALL #" + std::to_string(addrRepetitions[addr]);
                }

                // Display frame with comments if available
                if (stack.entries[i].comment[0]) {
                    _plugin_logprintf("%s %s (%s)%s\n", framePrefix.c_str(), frameText, stack.entries[i].comment, marker.c_str());
                } else {
                    _plugin_logprintf("%s %s%s\n", framePrefix.c_str(), frameText, marker.c_str());
                }
            }

            if (stack.total > 20) {
                _plugin_logprintf("║ ... plus %d more frames\n", stack.total - 20);
            }

            // Additional insights section
            if (type == StackIssueType::StackOverflow || type == StackIssueType::BufferOverflow) {
                // Look for recursive patterns
                int maxRepetitions = 0;
                std::string mostFrequentModule = "";

                for (auto& pair : moduleRepetitions) {
                    if (pair.second > maxRepetitions) {
                        maxRepetitions = pair.second;
                        mostFrequentModule = pair.first;
                    }
                }

                _plugin_logprintf("╠══════════════════════════════════════════════════════════════════════════════\n");
                _plugin_logprintf("║ 🔬 ADDITIONAL INSIGHTS:\n");

                if (maxRepetitions > 3) {
                    _plugin_logprintf("║ • Module '%s' appears %d times in the call stack - possible recursion\n",
                                    mostFrequentModule.c_str(), maxRepetitions);
                    _plugin_logprintf("║   This may indicate uncontrolled recursion, which can quickly exhaust stack space.\n");
                }

                // Provide advice based on issue type
                switch (type) {
                    case StackIssueType::StackOverflow:
                        _plugin_logprintf("║ • Suggestions for fixing stack overflows:\n");
                        _plugin_logprintf("║   - Check for infinite or excessive recursion\n");
                        _plugin_logprintf("║   - Reduce large local variable arrays (use heap instead)\n");
                        _plugin_logprintf("║   - Add base case checks in recursive functions\n");
                        break;
                    case StackIssueType::BufferOverflow:
                        _plugin_logprintf("║ • Suggestions for fixing buffer overflows:\n");
                        _plugin_logprintf("║   - Use safer string/memory functions (strncpy vs strcpy)\n");
                        _plugin_logprintf("║   - Add bounds checking in loops and array accesses\n");
                        _plugin_logprintf("║   - Consider using stack protection options (/GS compiler flag)\n");
                        break;
                    case StackIssueType::ReturnAddressCorruption:
                        _plugin_logprintf("║ • Suggestions for fixing return address corruption:\n");
                        _plugin_logprintf("║   - Check for buffer overruns just before this function returns\n");
                        _plugin_logprintf("║   - Look for stack manipulation code that might be incorrect\n");
                        _plugin_logprintf("║   - Debug buffer operations using safe memory access bounds\n");
                        break;
                    default:
                        break;
                }
            }
        }
    }

    // Close the box
    _plugin_logprintf("╚══════════════════════════════════════════════════════════════════════════════\n\n");
}

// Helper method to get educational explanations for stack issues
std::string StackDetector::GetEducationalNoteForIssue(StackIssueType type)
{
    switch (type) {
        case StackIssueType::StackOverflow:
            return "Each function call uses stack memory for local variables and return addresses. "
                   "Stack overflow happens when you run out of this memory, usually due to deep recursion "
                   "or very large local arrays. The call stack has a fixed size (typically 1-8MB).";

        case StackIssueType::BufferOverflow:
            return "Buffer overflow occurs when a program writes data beyond the end of a buffer. "
                   "On the stack, this often happens with arrays or string operations without proper length checks. "
                   "This can corrupt other variables, return addresses, and cause crashes or security vulnerabilities.";

        case StackIssueType::UseAfterReturn:
            return "Use-after-return is when code accesses local variables of a function after that function "
                   "has returned. Since local variables exist on the stack and are cleaned up when a function returns, "
                   "accessing them later can cause unpredictable behavior as that memory may be reused.";

        case StackIssueType::ReturnAddressCorruption:
            return "Return addresses tell the program where to continue execution after a function finishes. "
                   "If this address is corrupted (often by a buffer overflow), the program will jump to an "
                   "invalid location when the function returns, typically causing a crash.";

        case StackIssueType::StackAlignment:
            return "Stack alignment errors occur when the stack pointer is not properly aligned according to "
                   "platform requirements (16 bytes for x64, 4 bytes for x86). Improper alignment can cause "
                   "crashes, especially with instructions that require aligned memory access.";

        case StackIssueType::StackFrameError:
            return "The stack frame tracks local variables and parameters for each function call. "
                   "Errors in stack frame management can corrupt function parameters, local variables, "
                   "or return addresses. This commonly happens with incorrect assembly code or compiler optimizations.";

        default:
            return "Unknown stack issue detected. Stack issues generally involve memory corruption, "
                   "improper memory access, or exhaustion of available stack space.";
    }
}
