#pragma once

#include <windows.h>
#include <vector>
#include <map>
#include <string>
#include "../pluginsdk/_plugins.h"
#include "../pluginsdk/_dbgfunctions.h"
#include "../pluginsdk/_scriptapi.h"

// Типы обнаруженных проблем со стеком
enum class StackIssueType {
    StackOverflow,           // Переполнение стека
    BufferOverflow,          // Переполнение буфера на стеке
    UseAfterReturn,          // Использование после возврата
    ReturnAddressCorruption, // Коррупция адреса возврата
    StackAlignment,          // Проблемы с выравниванием стека
    StackFrameError          // Ошибка фрейма стека
};

// Структура для хранения информации о проблеме со стеком
struct StackIssue {
    StackIssueType type;
    duint address;           // Адрес, где обнаружена проблема
    std::string description; // Описание проблемы
    duint stackPtr;          // Значение указателя стека
    duint basePtr;           // Значение базового указателя
    std::vector<duint> stackTrace; // Трассировка стека

    StackIssue() : type(StackIssueType::StackOverflow), address(0), stackPtr(0), basePtr(0) {}
};

// Класс детектора проблем со стеком
class StackDetector {
public:
    StackDetector();
    ~StackDetector();

    // Инициализация детектора
    bool Initialize();

    // Остановка детектора
    void Shutdown();

    // Обработчики событий отладчика
    void OnBreakpoint(duint address);
    void OnStep();
    void OnException(EXCEPTION_RECORD* exceptionRecord);

    // Получение обнаруженных проблем
    const std::vector<StackIssue>& GetIssues() const;

    // Очистка списка проблем
    void ClearIssues();

    // Включение/выключение автоматического анализа
    void SetAutoAnalysis(bool enabled);

    // Получение состояния автоматического анализа
    bool GetAutoAnalysis() const;

    // Ручное сканирование текущего стека
    bool ScanCurrentStack();

    // Сохранение настроек
    bool SaveSettings();

    // Загрузка настроек
    bool LoadSettings();

private:
    // Методы анализа
    bool CheckStackOverflow();
    bool CheckBufferOverflow();
    bool CheckReturnAddressCorruption();
    bool CheckStackAlignment();
    bool CheckFramePointers();

    // Получение трассировки стека
    std::vector<duint> GetStackTrace();

    // Сохранение обнаруженной проблемы
    void AddIssue(StackIssueType type, duint address, const std::string& description);

    // Helper method to provide educational explanations for different stack issues
    std::string GetEducationalNoteForIssue(StackIssueType type);

    // Сохраненные проблемы
    std::vector<StackIssue> m_issues;

    // Карта для отслеживания базовых указателей функций
    std::map<duint, duint> m_functionBasePointers;

    // Флаг автоматического анализа
    bool m_autoAnalysisEnabled;
};

// Глобальный экземпляр детектора
extern StackDetector* g_stackDetector;
